@@ -67,7 +67,9 @@
 #include "MagickCore/memory-private.h"
 #include "MagickCore/option.h"
 #include "MagickCore/pixel-accessor.h"
+#include "MagickCore/profile-private.h"
 #include "MagickCore/quantum-private.h"
+#include "MagickCore/resource_.h"
 #include "MagickCore/static.h"
 #include "MagickCore/string_.h"
 #include "MagickCore/string-private.h"
@@ -174,12 +176,8 @@ static MagickBooleanType ReadHEICColorProfile(Image *image,
       StringInfo
         *profile;
 
-      profile=BlobToStringInfo(color_profile,length);
-      if (profile != (StringInfo*) NULL)
-        {
-          (void) SetImageProfile(image,"icc",profile,exception);
-          profile=DestroyStringInfo(profile);
-        }
+      profile=BlobToProfileStringInfo("icc",color_profile,length,exception);
+      (void) SetImageProfilePrivate(image,profile,exception);
     }
   color_profile=(unsigned char *) RelinquishMagickMemory(color_profile);
   return(MagickTrue);
@@ -216,7 +214,7 @@ static MagickBooleanType ReadHEICExifProfile(Image *image,
   if ((MagickSizeType) length > GetBlobSize(image))
     ThrowBinaryException(CorruptImageError,"InsufficientImageDataInFile",
       image->filename);
-  exif_profile=AcquireStringInfo(length);
+  exif_profile=AcquireProfileStringInfo("exif",length,exception);
   error=heif_image_handle_get_metadata(image_handle,id,
     GetStringInfoDatum(exif_profile));
   if ((IsHEIFSuccess(image,&error,exception) != MagickFalse) && (length > 4))
@@ -244,7 +242,7 @@ static MagickBooleanType ReadHEICExifProfile(Image *image,
       */
       length=GetStringInfoLength(exif_profile);
       datum=GetStringInfoDatum(exif_profile);
-      if ((length > 2) && 
+      if ((length > 2) &&
           ((memcmp(datum,"\xff\xd8",2) == 0) ||
            (memcmp(datum,"\xff\xe1",2) == 0)) &&
            (memcmp(datum+length-2,"\xff\xd9",2) == 0))
@@ -255,10 +253,12 @@ static MagickBooleanType ReadHEICExifProfile(Image *image,
       if (offset < GetStringInfoLength(exif_profile))
         {
           (void) DestroyStringInfo(SplitStringInfo(exif_profile,offset));
-          (void) SetImageProfile(image,"exif",exif_profile,exception);
+          (void) SetImageProfilePrivate(image,exif_profile,exception);
+          exif_profile=(StringInfo *) NULL;
         }
     }
-  exif_profile=DestroyStringInfo(exif_profile);
+  if (exif_profile != (StringInfo *) NULL)
+    exif_profile=DestroyStringInfo(exif_profile);
   return(MagickTrue);
 }
 
@@ -302,12 +302,8 @@ static MagickBooleanType ReadHEICXMPProfile(Image *image,
       StringInfo
         *profile;
 
-      profile=BlobToStringInfo(xmp_profile,length);
-      if (profile != (StringInfo*) NULL)
-        {
-          (void) SetImageProfile(image,"xmp",profile,exception);
-          profile=DestroyStringInfo(profile);
-        }
+      profile=BlobToProfileStringInfo("xmp",xmp_profile,length,exception);
+      (void) SetImageProfilePrivate(image,profile,exception);
     }
   xmp_profile=(unsigned char *) RelinquishMagickMemory(xmp_profile);
   return(MagickTrue);
@@ -376,6 +372,33 @@ static MagickBooleanType ReadHEICImageHandle(const ImageInfo *image_info,
   if (status == MagickFalse)
     return(MagickFalse);
   decode_options=heif_decoding_options_alloc();
+#if LIBHEIF_NUMERIC_VERSION >= HEIC_COMPUTE_NUMERIC_VERSION(1,16,0)
+  {
+    const char
+      *option;
+
+    option=GetImageOption(image_info,"heic:chroma-upsampling");
+    if (option != (char *) NULL)
+      {
+        if (LocaleCompare(option,"nearest-neighbor") == 0)
+          {
+            decode_options->color_conversion_options.
+              only_use_preferred_chroma_algorithm=1;
+            decode_options->color_conversion_options.
+              preferred_chroma_upsampling_algorithm=
+              heif_chroma_upsampling_nearest_neighbor;
+          }
+        else if (LocaleCompare(option,"bilinear") == 0)
+          {
+            decode_options->color_conversion_options.
+              only_use_preferred_chroma_algorithm=1;
+            decode_options->color_conversion_options.
+              preferred_chroma_upsampling_algorithm=
+              heif_chroma_upsampling_bilinear;
+          }
+      }
+    }
+#endif
   if (preserve_orientation != MagickFalse)
     decode_options->ignore_transformations=1;
   chroma=heif_chroma_interleaved_RGB;
@@ -530,6 +553,9 @@ static Image *ReadHEICImage(const ImageInfo *image_info,
   Image
     *image;
 
+  int
+    max_size;
+
   MagickBooleanType
     status;
 
@@ -578,6 +604,10 @@ static Image *ReadHEICImage(const ImageInfo *image_info,
   heif_context=heif_context_alloc();
   if (heif_context == (struct heif_context *) NULL)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+  max_size=(int) MagickMin(MagickMin(GetMagickResourceLimit(WidthResource),
+    GetMagickResourceLimit(HeightResource)),INT_MAX);
+  if (max_size != INT_MAX)
+    heif_context_set_maximum_image_size_limit(heif_context,max_size);
   error=heif_context_read_from_file(heif_context,image->filename,
     (const struct heif_reading_options *) NULL);
   if (IsHEIFSuccess(image,&error,exception) == MagickFalse)
@@ -763,7 +793,7 @@ ModuleExport size_t RegisterHEICImage(void)
   entry->flags|=CoderDecoderSeekableStreamFlag;
   entry->flags^=CoderBlobSupportFlag;
   (void) RegisterMagickInfo(entry);
-#if LIBHEIF_NUMERIC_VERSION > HEIC_COMPUTE_NUMERIC_VERSION(1,6,2) 
+#if LIBHEIF_NUMERIC_VERSION >= HEIC_COMPUTE_NUMERIC_VERSION(1,7,0)
   entry=AcquireMagickInfo("HEIC","AVIF","AV1 Image File Format");
 #if defined(MAGICKCORE_HEIC_DELEGATE)
   if (heif_have_decoder_for_format(heif_compression_AV1))
@@ -804,7 +834,7 @@ ModuleExport size_t RegisterHEICImage(void)
 */
 ModuleExport void UnregisterHEICImage(void)
 {
-#if LIBHEIF_NUMERIC_VERSION > HEIC_COMPUTE_NUMERIC_VERSION(1,6,2)
+#if LIBHEIF_NUMERIC_VERSION >= HEIC_COMPUTE_NUMERIC_VERSION(1,7,0)
   (void) UnregisterMagickInfo("AVIF");
 #endif
   (void) UnregisterMagickInfo("HEIC");
@@ -1176,7 +1206,7 @@ static MagickBooleanType WriteHEICImage(const ImageInfo *image_info,
   Image *image,ExceptionInfo *exception)
 {
   MagickBooleanType
-#if LIBHEIF_NUMERIC_VERSION > HEIC_COMPUTE_NUMERIC_VERSION(1,6,2)
+#if LIBHEIF_NUMERIC_VERSION >= HEIC_COMPUTE_NUMERIC_VERSION(1,7,0)
     encode_avif,
 #endif
     status;
@@ -1210,12 +1240,15 @@ static MagickBooleanType WriteHEICImage(const ImageInfo *image_info,
     return(status);
   scene=0;
   heif_context=heif_context_alloc();
-#if LIBHEIF_NUMERIC_VERSION > HEIC_COMPUTE_NUMERIC_VERSION(1,6,2)
+#if LIBHEIF_NUMERIC_VERSION >= HEIC_COMPUTE_NUMERIC_VERSION(1,7,0)
   encode_avif=(LocaleCompare(image_info->magick,"AVIF") == 0) ? MagickTrue :
     MagickFalse;
 #endif
   do
   {
+    const char
+      *option;
+
     const StringInfo
       *profile;
 
@@ -1234,7 +1267,7 @@ static MagickBooleanType WriteHEICImage(const ImageInfo *image_info,
     /*
       Get encoder for the specified format.
     */
-#if LIBHEIF_NUMERIC_VERSION > HEIC_COMPUTE_NUMERIC_VERSION(1,6,2)
+#if LIBHEIF_NUMERIC_VERSION >= HEIC_COMPUTE_NUMERIC_VERSION(1,7,0)
     if (encode_avif != MagickFalse)
       error=heif_context_get_encoder_for_format(heif_context,
         heif_compression_AV1,&heif_encoder);
@@ -1309,31 +1342,53 @@ static MagickBooleanType WriteHEICImage(const ImageInfo *image_info,
     status=IsHEIFSuccess(image,&error,exception);
     if (status == MagickFalse)
       break;
-#if LIBHEIF_NUMERIC_VERSION > HEIC_COMPUTE_NUMERIC_VERSION(1,6,2)
-    if (encode_avif != MagickFalse)
+    option=GetImageOption(image_info,"heic:speed");
+    if (option != (char *) NULL)
       {
-        const char
-          *option;
-
-        option=GetImageOption(image_info,"heic:speed");
-        if (option != (char *) NULL)
+        error=heif_encoder_set_parameter(heif_encoder,"speed",option);
+        status=IsHEIFSuccess(image,&error,exception);
+        if (status == MagickFalse)
+          break;
+      }
+    option=GetImageOption(image_info,"heic:chroma");
+    if (option != (char *) NULL)
+      {
+        error=heif_encoder_set_parameter(heif_encoder,"chroma",option);
+        status=IsHEIFSuccess(image,&error,exception);
+        if (status == MagickFalse)
+          break;
+      }
+    options=heif_encoding_options_alloc();
+#if LIBHEIF_NUMERIC_VERSION >= HEIC_COMPUTE_NUMERIC_VERSION(1,16,0)
+    option=GetImageOption(image_info,"heic:chroma-downsampling");
+    if (option != (char *) NULL)
+      {
+        if (LocaleCompare(option,"nearest-neighbor") == 0)
           {
-            error=heif_encoder_set_parameter(heif_encoder,"speed",option);
-            status=IsHEIFSuccess(image,&error,exception);
-            if (status == MagickFalse)
-              break;
+            options->color_conversion_options.
+              only_use_preferred_chroma_algorithm=1;
+            options->color_conversion_options.
+              preferred_chroma_downsampling_algorithm=
+              heif_chroma_downsampling_nearest_neighbor;
           }
-        option=GetImageOption(image_info,"heic:chroma");
-        if (option != (char *) NULL)
+        else if (LocaleCompare(option,"average") == 0)
           {
-            error=heif_encoder_set_parameter(heif_encoder,"chroma",option);
-            status=IsHEIFSuccess(image,&error,exception);
-            if (status == MagickFalse)
-              break;
+            options->color_conversion_options.
+              only_use_preferred_chroma_algorithm=1;
+            options->color_conversion_options.
+              preferred_chroma_downsampling_algorithm=
+              heif_chroma_downsampling_average;
+          }
+        else if (LocaleCompare(option,"sharp-yuv") == 0)
+          {
+            options->color_conversion_options.
+              only_use_preferred_chroma_algorithm=1;
+            options->color_conversion_options.
+              preferred_chroma_downsampling_algorithm=
+              heif_chroma_downsampling_sharp_yuv;
           }
       }
 #endif
-    options=heif_encoding_options_alloc();
 #if LIBHEIF_NUMERIC_VERSION >= HEIC_COMPUTE_NUMERIC_VERSION(1,14,0)
     if (image->orientation != UndefinedOrientation)
       options->image_orientation=(enum heif_orientation) image->orientation;
